/**
 *
 * WapiConfig.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */
package com.infoblox.wapi.common;

import org.json.JSONObject;
import org.w3c.dom.Document;

import com.infoblox.wapi.converter.AbstractConverter;
import com.infoblox.wapi.converter.JSONConverter;
import com.infoblox.wapi.converter.StringConverter;
import com.infoblox.wapi.converter.XMLConverter;

/**
 * Configuration for the WapiPort
 */
public class WapiConfig {
    /** User name for the authentication */
    private String userName = ""; //$NON-NLS-1$

    /** User password for the authentication */
    private String userPassword = ""; //$NON-NLS-1$

    /** Trust all server certificates */
    private boolean trustAllServers = false;

    /** Key store file with the trusted certificates */
    private String trustFile = ""; //$NON-NLS-1$

    /** Password for the trust key store */
    private String trustFilePass = ""; //$NON-NLS-1$

    /** Request timeout */
    private int connectTimeout = 30000;

    /** Web API service URL */
    private String wapiURL = ""; //$NON-NLS-1$

    /**
     * Content Type of the request body. The format of the data defaults to
     * JSON.
     */
    private ContentType contentType = ContentType.JSON;

    /**
     * Data format for returned values. The format of the data defaults to JSON.
     */
    private ReturnType returnType = ReturnType.JSON;

    /**
     * @param url
     * @param login
     * @param password
     * @param trustAll
     */
    public WapiConfig(String url, String login, String password,
            boolean trustAll) {
        super();
        setTrustAllServers(trustAll);
        setWapiURL(url);
        setUserName(login);
        setUserPassword(password);
    }

    /**
     * Get the userName.
     * 
     * @return String
     */
    public String getUserName() {
        return userName;
    }

    /**
     * Set the userName.
     * 
     * @param newUserName
     *            String
     */
    public void setUserName(String newUserName) {
        userName = newUserName;
    }

    /**
     * Get the userPassword.
     * 
     * @return String
     */
    public String getUserPassword() {
        return userPassword;
    }

    /**
     * Set the userPassword.
     * 
     * @param newUserPassword
     *            String
     */
    public void setUserPassword(String newUserPassword) {
        userPassword = newUserPassword;
    }

    /**
     * Get the trustAllServers.
     * 
     * @return boolean
     */
    public boolean isTrustAllServers() {
        return trustAllServers;
    }

    /**
     * Set the trustAllServers.
     * 
     * @param newTrustAllServers
     *            boolean
     */
    public void setTrustAllServers(boolean newTrustAllServers) {
        trustAllServers = newTrustAllServers;
    }

    /**
     * Get the trustFile.
     * 
     * @return String
     */
    public String getTrustFile() {
        return trustFile;
    }

    /**
     * Set the trustFile.
     * 
     * @param newTrustFile
     *            String
     */
    public void setTrustFile(String newTrustFile) {
        trustFile = newTrustFile;
    }

    /**
     * Get the trustFilePass.
     * 
     * @return String
     */
    public String getTrustFilePass() {
        return trustFilePass;
    }

    /**
     * Set the trustFilePass.
     * 
     * @param newTrustFilePass
     *            String
     */
    public void setTrustFilePass(String newTrustFilePass) {
        trustFilePass = newTrustFilePass;
    }

    /**
     * Get the wapiURL.
     * 
     * @return String
     */
    public String getWapiURL() {
        return wapiURL;
    }

    /**
     * Set the wapiURL.
     * 
     * @param newWapiURL
     *            String
     */
    public void setWapiURL(String newWapiURL) {
        wapiURL = newWapiURL;
    }

    /**
     * Get the contentType.
     * 
     * @return ContentType
     */
    public ContentType getContentType() {
        return contentType;
    }

    /**
     * Set the contentType.
     * 
     * @param newContentType
     *            ContentType
     */
    public void setContentType(ContentType newContentType) {
        contentType = newContentType;
    }

    /**
     * Set the returnType.
     * 
     * @param newReturnType
     *            ReturnType
     */
    public void setReturnType(ReturnType newReturnType) {
        returnType = newReturnType;
    }

    /**
     * Get the returnType.
     * 
     * @return ReturnType
     */
    public ReturnType getReturnType() {
        return returnType;
    }

    /**
     * Set the connectTimeout.
     * 
     * @param newConnectTimeout
     * 
     */
    public void setConnectTimeout(int newConnectTimeout) {
        this.connectTimeout = newConnectTimeout;
    }

    /**
     * Get the connectTimeout.
     * 
     * @return int
     */
    public int getConnectTimeout() {
        return connectTimeout;
    }

    /**
     * Data format for returned values. When the server returns an error with
     * status code >= 400, the body is always in JSON format, irrespective of
     * the ReturnType
     */
    public enum ReturnType {
        /** JSON format. */
        JSON("json", JSONObject.class, new JSONConverter()), //$NON-NLS-1$
        /** XML format. */
        XML("xml", Document.class, new XMLConverter()); //$NON-NLS-1$

        /** String with name of the type. */
        private String type;

        /** Java class associated with this return type. */
        private Class<?> outputClass;

        /** Converter for this return type. */
        private AbstractConverter<?> converter;

        /**
         * @param text
         * @param newOutputClass
         * @param newConverter
         */
        ReturnType(String text, Class<?> newOutputClass,
                AbstractConverter<?> newConverter) {
            this.type = text;
            this.outputClass = newOutputClass;
            this.converter = newConverter;
        }

        /**
         * @param newClass
         * @return ReturnType based on input class.
         */
        public static ReturnType getInstanceByClass(Class<?> newClass) {
            for (ReturnType returnType : ReturnType.values()) {
                if (returnType.getOutputClass() == newClass) {
                    return returnType;
                }
            }
            throw new IllegalArgumentException("Invalid class was specified."); //$NON-NLS-1$
        }

        /**
         * @param newType
         * @return ReturnType based on input String with name of the Type.
         */
        public static ReturnType getType(String newType) {
            return ReturnType.valueOf(newType.toUpperCase());
        }

        /**
         * @return String with text for the ReturnType
         */
        public String getText() {
            return type;
        }

        /**
         * Get the outputClass.
         * 
         * @return Class
         */
        public Class<?> getOutputClass() {
            return outputClass;
        }

        /**
         * Get the converter.
         * 
         * @return AbstractConverter
         */
        public AbstractConverter<?> getConverter() {
            return converter;
        }
    }

    /**
     * Content Type of the request body. The body of the HTTP request contains
     * data for the PUT and POST requests only.
     */
    public enum ContentType {
        /** JSON format. */
        JSON("application/json", JSONObject.class, new JSONConverter()), //$NON-NLS-1$
        /** XML format. */
        XML("application/xml", Document.class, new XMLConverter()), //$NON-NLS-1$
        /**
         * Arguments to method encoded in body. This is the same as
         * specification after ?, but it can handle longer sequences and is
         * directly supported by HTML forms.
         */
        URLENCODED(
                "application/x-www-form-urlencoded", String.class, new StringConverter()); //$NON-NLS-1$

        /** String with name of the type. */
        private String type;

        /** Java class associated with this input content type. */
        private Class<?> inputClass;

        /** Converter for this content type. */
        private AbstractConverter<?> converter;

        /**
         * @param text
         * @param newInputClass
         * @param newConverter
         */
        ContentType(String text, Class<?> newInputClass,
                AbstractConverter<?> newConverter) {
            this.type = text;
            this.inputClass = newInputClass;
            this.converter = newConverter;
        }

        /**
         * @param newClass
         * @return ContentType based on input class.
         */
        public static ContentType getInstanceByClass(Class<?> newClass) {
            for (ContentType contentType : ContentType.values()) {
                if (contentType.getInputClass() == newClass) {
                    return contentType;
                }
            }
            throw new IllegalArgumentException("Invalid class was specified."); //$NON-NLS-1$
        }

        /**
         * @param newType
         * @return ContentType based on input String with name of the Type.
         */
        public static ContentType getType(String newType) {
            return ContentType.valueOf(newType.toUpperCase());
        }

        /**
         * @return String with text of the ContentType
         */
        public String getType() {
            return type;
        }

        /**
         * Get the inputClass.
         * 
         * @return Class
         */
        public Class<?> getInputClass() {
            return inputClass;
        }

        /**
         * Get the converter.
         * 
         * @return AbstractConverter
         */
        public AbstractConverter<?> getConverter() {
            return converter;
        }
    }
}