/**
 *
 * WapiPort.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */
package com.infoblox.wapi.common;

import java.io.DataOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;

import javax.net.ssl.HttpsURLConnection;

import org.apache.log4j.Logger;

import com.infoblox.wapi.common.WapiConfig.ContentType;
import com.infoblox.wapi.converter.AbstractConverter;

/**
 * Port for the Web API.
 */
public class WapiPort {

    /** Current instance */
    private static WapiPort instance;

    /** Configuration for the WapiPort */
    private WapiConfig config;

    /** True if all host names and certificates were trusted */
    private static boolean trusted = false;

    /** Log4j logger */
    private static Logger logger = Logger.getLogger(WapiPort.class);

    /**
     * Request methods enumeration
     */
    public enum RequestMethod {
        /** Submits data to be processed. */
        POST,
        /** Requests a representation of the specified resource. */
        GET,
        /** Uploads a representation of the specified resource. */
        PUT,
        /** Deletes the specified resource. */
        DELETE
    }

    /**
     * Default. Use getInstance() instead.
     */
    private WapiPort() {
        // Empty by design.
    }

    /**
     * @return An instance of the WapiPort.
     */
    public static WapiPort getInstance() {
        if (instance == null) {
            instance = new WapiPort();
        }
        return instance;
    }

    /**
     * @param newConfig
     */
    public void init(WapiConfig newConfig) {
        instance.config = newConfig;
        if (!trusted) {
            trustServers(newConfig);
            trusted = true;
        }
    }

    /**
     * @param path
     * @param requestMethod
     * @return WapiResponse with result of the request.
     */
    public WapiResponse sendRequest(String path, RequestMethod requestMethod) {
        return sendRequest(path, requestMethod, ""); //$NON-NLS-1$
    }

    /**
     * @param path
     * @param requestMethod
     * @param input
     * @return WapiResponse with result of the request.
     */
    public WapiResponse sendRequest(String path, RequestMethod requestMethod,
            String input) {
        return sendRequest(path, "", requestMethod, input); //$NON-NLS-1$
    }

    /**
     * @param path
     * @param params
     * @param requestMethod
     * @return WapiResponse with result of the request.
     */
    public WapiResponse sendRequest(String path, String params,
            RequestMethod requestMethod) {
        return sendRequest(path, params, requestMethod, null);
    }

    /**
     * @param path
     * @param params
     * @param requestMethod
     * @param input
     * @return WapiResponse with result of the request.
     */
    public WapiResponse sendRequest(String path, String params,
            RequestMethod requestMethod, String input) {
        if (getConfig() == null) {
            throw new RuntimeException("WapiPort is not initialized."); //$NON-NLS-1$
        }
        String address = getConfig().getWapiURL() + path;

        String parameters = "_return_type=" + getConfig().getReturnType().getText(); //$NON-NLS-1$
        if (null != params && !params.isEmpty()) {
            parameters += "&" + params; //$NON-NLS-1$
        }

        String inputBody = ""; //$NON-NLS-1$
        if (null != input && !input.isEmpty()) {
            inputBody += input;
        }
        // In case of urlencoded content type and POST or PUT,
        // add parameters to the body.
        if ((RequestMethod.POST.equals(requestMethod) ||
        		RequestMethod.PUT.equals(requestMethod)) &&
        	  ContentType.URLENCODED.equals(getConfig().getContentType())) {
            if (!inputBody.isEmpty()) {
                inputBody += "&"; //$NON-NLS-1$
            }
            inputBody += parameters;
        } else {
            // In other cases, add parameters to the address.
            address += "?" + parameters; //$NON-NLS-1$
        }

        try {
            URL url = new URL(address);
            // Encode authorization information.
            String encoding = new sun.misc.BASE64Encoder().encode((getConfig()
                    .getUserName() + ":" + getConfig() //$NON-NLS-1$
                    .getUserPassword()).getBytes());
            HttpsURLConnection connection = (HttpsURLConnection) url
                    .openConnection();
            connection.setRequestProperty("Authorization", "Basic " + encoding); //$NON-NLS-1$ //$NON-NLS-2$
            connection.setRequestProperty("Content-Type", getConfig() //$NON-NLS-1$
                    .getContentType().getType());
            connection.setRequestMethod(requestMethod.toString());
            connection.setDoOutput(true);
            connection.setConnectTimeout(getConfig().getConnectTimeout());
            // In case of POST and PUT request methods,
            // request should has input body.
            if ((RequestMethod.POST.equals(requestMethod) || RequestMethod.PUT
                    .equals(requestMethod)) && !inputBody.isEmpty()) {
                connection.setDoInput(true);
                DataOutputStream requestStream = new DataOutputStream(
                        connection.getOutputStream());
                requestStream.writeBytes(inputBody);
                requestStream.flush();
                requestStream.close();
            }

            boolean success = (200 <= connection.getResponseCode() && connection
                    .getResponseCode() < 300);
            String response = AbstractConverter
                    .streamToString(success ? connection.getInputStream()
                            : connection.getErrorStream());

            return new WapiResponse(connection.getResponseCode(),
                    connection.getResponseMessage(), response);
        } catch (MalformedURLException e) {
            logger.error("Exception occurred while creating URL", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        } catch (IOException e) {
            logger.error("Exception occurred while connection setup", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
    }

    /**
     * @param newConfig
     */
    public static void trustServers(WapiConfig newConfig) {
        if (newConfig.isTrustAllServers()) {
            WapiUtil.trustAllHostNames();
            WapiUtil.trustHttpsCertificates(WapiUtil.getTrustAllManagers());

        } else {
            WapiUtil.trustHttpsCertificates(WapiUtil
                    .getTrustManagers(newConfig));
        }
    }

    /**
     * @return WapiConfig with configuration of the WapiPort
     */
    public WapiConfig getConfig() {
        return config;
    }
}
