/**
 *
 * Sample.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */

package com.infoblox.wapi.java.client.sample;

import org.apache.log4j.Logger;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import org.w3c.dom.Document;

import com.infoblox.wapi.common.WapiConfig;
import com.infoblox.wapi.common.WapiPort;
import com.infoblox.wapi.converter.AbstractConverter;
import com.infoblox.wapi.converter.JSONConverter;
import com.infoblox.wapi.dao.IWapiDAO;
import com.infoblox.wapi.dao.WapiDAO;

/**
 * Java sample code for Infoblox Web API.
 */
public class Sample {
    /** URL for the Web API server */
    private static final String URL = "https://192.168.1.2/wapi/v1.0/"; //$NON-NLS-1$

    /** User login */
    private static final String LOGIN = "admin"; //$NON-NLS-1$

    /** User password */
    private static final String PASSWORD = "infoblox"; //$NON-NLS-1$

    /** Log4j logger */
    private static Logger logger = Logger.getLogger(Sample.class);

    /**
     * @return WapiPort Default configuration for the sample.
     */
    private static WapiConfig createDefaultConfig() {
        return new WapiConfig(URL, LOGIN, PASSWORD, true);
    }

    /**
     * Perform some operations with Networks with JSON.
     * 
     * @param port
     */
    public static void exampleJSON(WapiPort port) {
        IWapiDAO<JSONObject, JSONObject> network = new WapiDAO<JSONObject, JSONObject>(
                port, "network", JSONObject.class, JSONObject.class); //$NON-NLS-1$
        try {
            JSONObject object = new JSONObject();
            object.put("network", "10.45.0.0/24"); //$NON-NLS-1$ //$NON-NLS-2$

            String ref = network.create(object);

            JSONObject fieldsToModify = new JSONObject();
            fieldsToModify.put("comment", "new one"); //$NON-NLS-1$ //$NON-NLS-2$

            network.update(ref, fieldsToModify);

            JSONObject fieldsToSearch = new JSONObject();
            fieldsToSearch.put("comment", "new one"); //$NON-NLS-1$ //$NON-NLS-2$
            System.out.println("found by object:"); //$NON-NLS-1$
            JSONArray array = network.search(fieldsToSearch).getJSONArray(
                    JSONConverter.STORE_FIELD);
            for (int i = 0; i < array.length(); i++) {
                System.out.println(array.get(i));
            }

            System.out.println("read all objects:"); //$NON-NLS-1$
            array = network.readAll().getJSONArray(JSONConverter.STORE_FIELD);
            for (int i = 0; i < array.length(); i++) {
                System.out.println(array.get(i));
            }

            // Perform search Networks operation with String.
            exampleString(port);

            System.out.println("deleted:\n" + network.delete(ref)); //$NON-NLS-1$

            JSONArray arrayAfterDelete = network.readAll().getJSONArray(
                    JSONConverter.STORE_FIELD);
            for (int i = 0; i < arrayAfterDelete.length(); i++) {
                System.out.println(arrayAfterDelete.get(i));
            }
        } catch (JSONException e) {
            logger.error("An exception occurred when creating JSON", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
    }

    /**
     * Perform search Networks operation with String.
     * 
     * @param port
     */
    public static void exampleString(WapiPort port) {
        IWapiDAO<String, JSONObject> daoForSearch = new WapiDAO<String, JSONObject>(
                port, "network", String.class, JSONObject.class); //$NON-NLS-1$

        System.out.println("found by string:"); //$NON-NLS-1$
        try {
            JSONArray array = daoForSearch
                    .search("comment=new one").getJSONArray(JSONConverter.STORE_FIELD); //$NON-NLS-1$ 
            for (int i = 0; i < array.length(); i++) {

                System.out.println(array.get(i));

            }
        } catch (JSONException e) {
            logger.error(
                    "An exception occurred when getting object from JSONArray", e); //$NON-NLS-1$
            throw new RuntimeException(e);
        }
    }

    /**
     * Perform some operations with Networks with XML.
     * 
     * @param port
     */
    public static void exampleXML(WapiPort port) {
        IWapiDAO<Document, Document> record = new WapiDAO<Document, Document>(
                port, "record:ptr", Document.class, Document.class); //$NON-NLS-1$
        Document document = record.readAll();
        System.out.println(AbstractConverter.documentToString(document));
    }

    /**
     * Run sample code.
     * 
     * @param args
     */
    public static void main(String[] args) {
        WapiConfig config = Sample.createDefaultConfig();
        WapiPort port = WapiPort.getInstance();
        port.init(config);
        exampleJSON(port);
        exampleXML(port);
    }

}
