/**
 *
 * WapiJavaClient.java
 *
 * Copyright 2012 Infoblox Inc. All rights reserved.
 *             
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     1. Redistributions of source code must retain the above copyright notice,
 *     this list of conditions and the following disclaimer.
 *     2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY INFOBLOX AND CONTRIBUTORS ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL INFOBLOX OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * The views and conclusions contained in the software and documentation are
 * those of the authors and should not be interpreted as representing official
 * policies, either expressed or implied, of Infoblox.
 *
 */
package com.infoblox.wapi.java.client.sample;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.HelpFormatter;
import org.apache.commons.cli.OptionBuilder;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;
import org.apache.log4j.Logger;

import com.infoblox.wapi.common.WapiConfig;
import com.infoblox.wapi.common.WapiConfig.ContentType;
import com.infoblox.wapi.common.WapiConfig.ReturnType;
import com.infoblox.wapi.common.WapiPort;
import com.infoblox.wapi.dao.IWapiDAO;
import com.infoblox.wapi.dao.WapiDAO;

/**
 * Main class for Infoblox Web API sample code.
 */
public class WapiJavaClient {

    /** String with example. */
    private static final String EXAMPLE_TEXT = "Example: " + //$NON-NLS-1$
            "-a 192.168.1.2 -u user -p testpw -o readall -t network"; //$NON-NLS-1$

    /** Log4j logger */
    private static Logger logger = Logger.getLogger(WapiJavaClient.class);

    /**
     * @param args
     */
    public static void main(String[] args) {

        // Create the command line parser and options.
        CommandLineParser parser = new PosixParser();
        Options helpOptions = createHelpOptions();
        Options generalOptions = createGeneralOptions();

        try {
            CommandLine line = parser.parse(helpOptions, args, true);

            if (line.hasOption('h')) {

                // Print the help.
                printHelp(helpOptions, generalOptions);
            } else {

                // Parse the general command line arguments.
                line = parser.parse(generalOptions, args);

                // Use 1.0 as default version.
                String version = "1.0"; //$NON-NLS-1$
                if (line.hasOption("version")) { //$NON-NLS-1$
                    version = line.getOptionValue("version"); //$NON-NLS-1$
                }

                String url = "https://" + //$NON-NLS-1$
                        line.getOptionValue("host-address") //$NON-NLS-1$
                        + "/wapi/v" + version + "/"; //$NON-NLS-1$ //$NON-NLS-2$
                String login = line.getOptionValue("user"); //$NON-NLS-1$
                String password = line.getOptionValue("password"); //$NON-NLS-1$

                // Use urlencoded as default input type.
                String inputType = "urlencoded"; //$NON-NLS-1$
                if (line.hasOption("input")) { //$NON-NLS-1$
                    inputType = line.getOptionValue("input"); //$NON-NLS-1$
                }

                // And Use JSON as default output type.
                String outputType = "json"; //$NON-NLS-1$
                if (line.hasOption("output")) { //$NON-NLS-1$
                    outputType = line.getOptionValue("output"); //$NON-NLS-1$
                }

                String operation = line.getOptionValue("operation"); //$NON-NLS-1$
                String params = ""; //$NON-NLS-1$
                if (line.hasOption("data")) { //$NON-NLS-1$
                    params = line.getOptionValue("data"); //$NON-NLS-1$
                }

                String objectType = line.getOptionValue("object-type"); //$NON-NLS-1$
                if (line.hasOption("ref")) { //$NON-NLS-1$
                    objectType = line.getOptionValue("ref"); //$NON-NLS-1$
                }

                WapiConfig config = new WapiConfig(url, login, password, true);
                WapiPort port = WapiPort.getInstance();
                port.init(config);
                performOperation(port, objectType, inputType, outputType,
                        operation, params);
            }

        } catch (ParseException e) {
            logger.error("Unexpected parse exception: ", e); //$NON-NLS-1$
            System.out.println("Unexpected parse exception: " + e.getMessage()); //$NON-NLS-1$
        }
    }

    /**
     * Print the help.
     * 
     * @param helpOptions
     * @param generalOptions
     */
    private static void printHelp(Options helpOptions, Options generalOptions) {
        HelpFormatter formatter = new HelpFormatter();
        formatter.setSyntaxPrefix("Web API java sample "); //$NON-NLS-1$
        formatter.printHelp("help options:", helpOptions); //$NON-NLS-1$
        formatter.printHelp("general options:", "", //$NON-NLS-1$ //$NON-NLS-2$
                generalOptions, EXAMPLE_TEXT);
    }

    /**
     * Create the help options.
     * 
     * @return Options
     */
    private static Options createHelpOptions() {
        Options helpOptions = new Options();
        helpOptions.addOption("h", "help", false, //$NON-NLS-1$ //$NON-NLS-2$
                "Print help for this application."); //$NON-NLS-1$
        return helpOptions;
    }

    /**
     * Create the general command line options.
     * 
     * @return Options
     */
    @SuppressWarnings("static-access")
    private static Options createGeneralOptions() {
        Options options = new Options();
        options.addOption(OptionBuilder
                .hasArg(true)
                .withLongOpt("password") //$NON-NLS-1$
                .isRequired()
                .withDescription(
                        "Required argument: User password for the authentication.") //$NON-NLS-1$
                .create("p")); //$NON-NLS-1$ 
        options.addOption(OptionBuilder
                .hasArg(true)
                .withLongOpt("user") //$NON-NLS-1$
                .isRequired()
                .withDescription(
                        "Required argument: User name for the authentication.") //$NON-NLS-1$
                .create("u")); //$NON-NLS-1$ 
        options.addOption(OptionBuilder.hasArg(true)
                .withLongOpt("host-address") //$NON-NLS-1$
                .isRequired()
                .withDescription(
                        "Required argument: Web API service host address.") //$NON-NLS-1$
                .create("a")); //$NON-NLS-1$
        options.addOption(OptionBuilder
                .hasArg(true)
                .withLongOpt("operation") //$NON-NLS-1$
                .isRequired()
                .withDescription(
                        "Required argument: Operation with WAPI objects: create, read, readall, update, delete or search.") //$NON-NLS-1$
                .create("o")); //$NON-NLS-1$ 
        options.addOption(OptionBuilder.hasArg(true)
                .withLongOpt("data") //$NON-NLS-1$
                .withDescription(
                        "Optional argument: input data such as search criteria or fields of new object.") //$NON-NLS-1$
                .create("d")); //$NON-NLS-1$ 
        options.addOption(OptionBuilder
                .hasArg(true)
                .withDescription(
                        "Optional argument: Input type: json, xml, urlencoded(default).") //$NON-NLS-1$
                .create("input")); //$NON-NLS-1$
        options.addOption(OptionBuilder
                .hasArg(true)
                .withDescription(
                        "Optional argument: Return type: json(default), xml.") //$NON-NLS-1$
                .create("output")); //$NON-NLS-1$
        options.addOption(OptionBuilder.hasArg(true)
                .withLongOpt("version") //$NON-NLS-1$
                .withDescription(
                        "Optional argument: Version of WAPI service. Default is 2.0.") //$NON-NLS-1$
                .create("v")); //$NON-NLS-1$
        options.addOption(OptionBuilder.hasArg(true).withLongOpt("object-type") //$NON-NLS-1$
                .isRequired()
                .withDescription("Required argument: Type of Wapi object.") //$NON-NLS-1$
                .create("t")); //$NON-NLS-1$
        options.addOption(OptionBuilder.hasArg(true).withLongOpt("ref") //$NON-NLS-1$
                .withDescription("Optional argument: Ref to the Wapi object.") //$NON-NLS-1$
                .create("r")); //$NON-NLS-1$
        return options;
    }

    /**
     * @param newPort
     * @param newObjectType
     * @param inputType
     * @param outputType
     * @param newOperation
     * @param newParams
     */
    @SuppressWarnings({ "unchecked", "rawtypes" })
    private static void performOperation(WapiPort newPort,
            String newObjectType, String inputType, String outputType,
            String newOperation, String newParams) {

        ContentType contentType = ContentType.getType(inputType);
        ReturnType returnType = ReturnType.getType(outputType);
        // Object type will be always set in the URL
        IWapiDAO dao = new WapiDAO(newPort, newObjectType,
                contentType.getInputClass(), returnType.getOutputClass());
        OperationType operation = OperationType.getOperation(newOperation);
        System.out.println(operation.performOperation(dao, newParams));
    }

}
